// Copyright Supranational LLC
// Licensed under the Apache License, Version 2.0, see LICENSE for details.
// SPDX-License-Identifier: Apache-2.0

#ifndef __RUSTERROR_H__
#define __RUSTERROR_H__

#ifdef __cplusplus
# include <string>
# include <cstring>
#else
# include <string.h>
#endif

struct RustError { /* to be returned exclusively by value */
    int code;
    char *message;
#ifdef __cplusplus
    RustError(int e = 0) : code(e)
    {   message = nullptr;   }
    RustError(int e, const std::string& str) : code(e)
    {   message = str.empty() ? nullptr : strdup(str.c_str());   }
    RustError(int e, const char *str) : code(e)
    {   message = str==nullptr ? nullptr : strdup(str);   }
    // no destructor[!], Rust takes care of the |message|
#endif
};
#ifndef __cplusplus
typedef struct RustError RustError;
#endif

#define RUST_OK(expr) do {                                  \
    RustError e = expr;                                     \
    if (e.code != cudaSuccess) {                            \
        throw cuda_error((cudaError_t)e.code, e.message);   \
    }                                                       \
} while(0)

#endif
